from dotenv import load_dotenv
from rich import print
import requests
import os

load_dotenv()

BASE_URL = os.getenv("BASE_URL")


class BaseAPI:
    """
    BaseAPI provides basic HTTP methods (_get, _post, _patch, _delete) for interacting with a RESTful API.
    All other API classes inherit from this to perform CRUD operations.
    """

    def __init__(self, base_url=BASE_URL):
        """
        Initialize the API with a base URL.
        :param base_url: The root URL for the API endpoints.
        """
        self.base_url = base_url

    def _get(self, endpoint):
        """
        Send a GET request to the specified endpoint.
        :param endpoint: API endpoint (string)
        :return: Response object
        """
        return requests.get(f"{self.base_url}{endpoint}")

    def _post(self, endpoint, data):
        """
        Send a POST request to the specified endpoint with JSON data.
        :param endpoint: API endpoint (string)
        :param data: Data to send (dict)
        :return: Response object
        """
        return requests.post(f"{self.base_url}{endpoint}", json=data)

    def _patch(self, endpoint, data):
        """
        Send a PATCH request to the specified endpoint with JSON data.
        :param endpoint: API endpoint (string)
        :param data: Data to update (dict)
        :return: Response object
        """
        return requests.patch(f"{self.base_url}{endpoint}", json=data)

    def _delete(self, endpoint, headers=None):
        """
        Send a DELETE request to the specified endpoint.
        :param endpoint: API endpoint (string)
        :param headers: Optional headers (dict)
        :return: JSON response or error message
        """
        url = f"{self.base_url}{endpoint}"
        try:
            response = requests.delete(url, headers=headers)
            response.raise_for_status()
            try:
                return response.json()
            except ValueError:
                return {"status": "success", "message": "Deleted", "status_code": response.status_code}
        except requests.RequestException as e:
            print(f"[red]DELETE request failed for {url}: {e}[/red]")
            return {"status": "error", "message": str(e)}


class BookAPI(BaseAPI):
    """
    BookAPI provides methods to create, update, delete, and retrieve books via the API.
    Inherits from BaseAPI.
    """

    def create(self, book_data: dict):
        """
        Create a new book entry in the API.
        :param book_data: Dictionary containing book details.
        :return: Book ID if successful, or error message.
        """
        data = {
            "book_id": book_data['book_id'],
            "title": book_data['title'],
            "author": book_data['author'],
            "author_id": book_data['author_id'],
            "category": ','.join(book_data['category']),
            "category_id": book_data['category_id'],
            "book_link": book_data['url'],
            "content": book_data['content'],
        }
        response = self._post("books/", data)
        try:
            resp_json = response.json()
        except Exception:
            resp_json = {}
        if hasattr(response, 'status_code') and response.status_code == 201 and resp_json.get('book_id'):
            print(
                f"Book created successfully. Title: {data.get('title')}, ID: {resp_json.get('book_id')}")
            return resp_json.get('book_id', '')
        else:
            error_msg = resp_json.get('message', response.text if hasattr(
                response, 'text') else 'Unknown error')
            print(f"Failed to create book: {error_msg}")
            return {"success": False, "message": error_msg}

    def update(self, book_id, updated_data):
        """
        Update an existing book entry.
        :param book_id: ID of the book to update.
        :param updated_data: Dictionary of fields to update.
        :return: Success or error message.
        """
        response = self._patch(f"books/{book_id}/", updated_data)
        if isinstance(response, dict) and response.get("status") == "error":
            print(
                f"Failed to update book {book_id}: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        print(f"Book {book_id} updated successfully.")
        return {"success": True, "message": f"Book {book_id} updated."}

    def delete(self, book_id):
        """
        Delete a book entry by its ID.
        :param book_id: ID of the book to delete.
        :return: Success or error message.
        """
        response = self._delete(f"books/{book_id}/")
        if isinstance(response, dict) and response.get("status") == "error":
            print(
                f"Failed to delete book {book_id}: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        print(f"Book {book_id} deleted successfully.")
        return {"success": True, "message": f"Book {book_id} deleted."}

    def get(self, book_id=None):
        """
        Retrieve a book by ID, or all books if no ID is provided.
        :param book_id: (Optional) ID of the book.
        :return: Book data or list of books.
        """
        if book_id:
            data = self._get(f"books/{book_id}/").json()
            if isinstance(data, dict) and "content" in data:
                data.pop("content")
            return data
        data = self._get("books/").json()
        if isinstance(data, list):
            for book in data:
                if "content" in book:
                    book.pop("content")
        return data

    def get_all_books(self, is_posted=False):
        """
        Retrieve all books, optionally filtering by 'is_posted' status.
        :param is_posted: Boolean to filter books.
        :return: List of books.
        """
        response = self._get("books/")
        if response.status_code == 200:
            books = [book for book in response.json(
            ) if book.get('is_posted') == is_posted]
            if len(books) == 0:
                print('All books are up to date')
                return []
            print("📚 All Books:")
            for book in books:
                print(
                    f"ID: {book.get('book_id')}, Title: {book.get('title')}, Author: {book.get('author')}")
        else:
            print(f"❌ Failed to retrieve books: {response.status_code}")
            print("Response:", response.text)
            return []
        return books


class AuthorAPI(BaseAPI):
    """
    AuthorAPI provides methods to create, update, delete, and retrieve authors via the API.
    Inherits from BaseAPI.
    """

    def create(self, author_data):
        """
        Create a new author entry in the API.
        :param author_data: Dictionary containing author details.
        :return: Author ID if successful, or error message.
        """
        author = {
            "author_id": author_data['id'],
            "author_name": author_data['full_name'],
            "author_link": author_data['author_link']
        }
        
        response = self._post("authors/", author)
        if isinstance(response, dict) and response.get("status") == "error":
            print(f"Failed to create author: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        obj_id = None
        try:
            obj_id = response.json().get("id", "")
        except Exception:
            pass
        print(
            f"Author created successfully. Name: {author.get('author_name')}")
        return {"success": True, "message": f"Author '{author.get('name', author.get('author'))}' created.", "id": obj_id}

    def update(self, author_id, updated_data):
        """
        Update an existing author entry.
        :param author_id: ID of the author to update.
        :param updated_data: Dictionary of fields to update.
        :return: Success or error message.
        """
        response = self._patch(f"authors/{author_id}/", updated_data)
        if isinstance(response, dict) and response.get("status") == "error":
            print(
                f"Failed to update author {author_id}: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        print(f"Author updated successfully.")
        return {"success": True, "message": f"Author {author_id} updated."}

    def delete(self, author_id):
        """
        Delete an author entry by its ID.
        :param author_id: ID of the author to delete.
        :return: Success or error message.
        """
        response = self._delete(f"authors/{author_id}/")
        if isinstance(response, dict) and response.get("status") == "error":
            print(
                f"Failed to delete author {author_id}: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        print(f"Author {author_id} deleted successfully.")
        return {"success": True, "message": f"Author {author_id} deleted."}

    def get(self, author_id=None):
        """
        Retrieve an author by ID, or all authors if no ID is provided.
        :param author_id: (Optional) ID of the author.
        :return: Author data or list of authors.
        """
        if author_id:
            return self._get(f"authors/{author_id}/").json()
        return self._get("authors/").json()

    def get_all_authors(self):
        """
        Retrieve all authors from the API.
        :return: List of authors.
        """
        response = self._get("authors/")
        if response.status_code == 200:
            authors = response.json()
            print(f"Authors found: {len(authors)}")
            return authors
        else:
            print(f"❌ Failed to retrieve authors: {response.status_code}")
            print("Response:", response.text)
            return []

    def get_unscraped_authors(self):
        """
        Retrieve all authors from the API that are not marked as scraped.
        :return: List of unscraped authors.
        """
        response = self._get("authors/")
        if response.status_code == 200:
            authors = response.json()
            unscraped = [a for a in authors if a.get('is_scraped') == "no"]
            print(f"Unscraped authors found: {len(unscraped)}")
            return unscraped
        else:
            print(f"❌ Failed to retrieve authors: {response.status_code}")
            print("Response:", response.text)
            return []


class TokenAPI(BaseAPI):
    """
    TokenAPI provides methods to create, update, and delete user tokens via the API.
    Inherits from BaseAPI.
    """

    def create(self, token_data):
        """
        Create a new user token entry in the API.
        :param token_data: Dictionary containing token details.
        :return: Token ID if successful, or error message.
        """
        response = self._post("users/", token_data)
        if isinstance(response, dict) and response.get("status") == "error":
            print(f"Failed to create token: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        obj_id = None
        try:
            obj_id = response.json().get("id", "")
        except Exception:
            pass
        print(
            f"Token created successfully for user: {token_data.get('username', token_data.get('user_id'))}")
        return {"success": True, "message": f"Token for user '{token_data.get('username', token_data.get('user_id'))}' created.", "id": obj_id}

    def update(self, user_id, updated_data):
        """
        Update an existing user token entry.
        :param user_id: ID of the user whose token to update.
        :param updated_data: Dictionary of fields to update.
        :return: Success or error message.
        """
        response = self._patch(f"users/{user_id}/", updated_data)
        if isinstance(response, dict) and response.get("status") == "error":
            print(
                f"Failed to update token for user {user_id}: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        print(f"Token for user {user_id} updated successfully.")
        return {"success": True, "message": f"Token for user {user_id} updated."}
    
    def get_all_tokens(self):
        """
        Retrieve all user tokens from the API.
        :return: List of user tokens.
        """
        response = self._get("users/")
        
        if response.status_code == 200:
            tokens = [token.get('token') for token in response.json()]
            return tokens
        else:
            print(f"❌ Failed to retrieve tokens: {response.status_code}")
            print("Response:", response.text)
            return []

    def delete(self, user_id):
        """
        Delete a user token entry by its ID.
        :param user_id: ID of the user whose token to delete.
        :return: Success or error message.
        """
        response = self._delete(f"users/{user_id}/")
        if isinstance(response, dict) and response.get("status") == "error":
            print(
                f"Failed to delete token for user {user_id}: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        print(f"Token for user {user_id} deleted successfully.")
        return {"success": True, "message": f"Token for user {user_id} deleted."}

    def get(self, user_id=None):
        """
        Retrieve a user by ID, or all users if no ID is provided.
        :param user_id: (Optional) ID of the user.
        :return: User data or list of users.
        """
        import random
        if user_id:
            return self._get(f"users/{user_id}/").json()
        data = self._get("users/").json()
        if isinstance(data, list):
            random.shuffle(data)
        return data


class CategoryAPI(BaseAPI):
    """
    CategoryAPI provides methods to create, update, and delete categories via the API.
    Inherits from BaseAPI.
    """

    def create(self, category_data):
        """
        Create a new category entry in the API.
        :param category_data: Dictionary containing category details.
        :return: Category ID if successful, or error message.
        """
        response = self._post("categories/", category_data)
        if isinstance(response, dict) and response.get("status") == "error":
            print(f"Failed to create category: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        obj_id = None
        try:
            obj_id = response.json().get("id", "")
        except Exception:
            pass
        print(
            f"Category created successfully. Name: {category_data.get('name', category_data.get('category'))}")
        return {"success": True, "message": f"Category '{category_data.get('name', category_data.get('category'))}' created.", "id": obj_id}

    def update(self, category_id, updated_data):
        """
        Update an existing category entry.
        :param category_id: ID of the category to update.
        :param updated_data: Dictionary of fields to update.
        :return: Success or error message.
        """
        response = self._patch(f"categories/{category_id}/", updated_data)
        if isinstance(response, dict) and response.get("status") == "error":
            print(
                f"Failed to update category {category_id}: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        print(f"Category {category_id} updated successfully.")
        return {"success": True, "message": f"Category {category_id} updated."}

    def delete(self, category_id):
        """
        Delete a category entry by its ID.
        :param category_id: ID of the category to delete.
        :return: Success or error message.
        """
        response = self._delete(f"categories/{category_id}/")
        if isinstance(response, dict) and response.get("status") == "error":
            print(
                f"Failed to delete category {category_id}: {response.get('message')}")
            return {"success": False, "message": response.get("message")}
        print(f"Category {category_id} deleted successfully.")
        return {"success": True, "message": f"Category {category_id} deleted."}

    def get(self, category_id=None):
        """
        Retrieve a category by ID, or all categories if no ID is provided.
        :param category_id: (Optional) ID of the category.
        :return: Category data or list of categories.
        """
        if category_id:
            return self._get(f"categories/{category_id}/").json()
        return self._get("categories/").json()



import requests
import json
from selectolax.parser import HTMLParser
import importlib.resources


class BookScraper:
    """
    BookScraper is responsible for scraping book data from ebanglalibrary.com.
    It provides methods to fetch book lists, details, content, and category IDs.
    """

    def __init__(self, single_page_cate: list):
        """
        Initialize the BookScraper with a list of single-page category IDs.

        Args:
            single_page_cate (list): List of category IDs that are single-page.
        """
        self.headers = {
            "accept": "*/*",
            "accept-language": "en-BD,en-US;q=0.9,en;q=0.8",
            "priority": "u=1, i",
            "referer": "https://www.ebanglalibrary.com/",
            "sec-ch-ua": '"Google Chrome";v="137", "Chromium";v="137", "Not/A)Brand";v="24"',
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/137.0.0.0 Safari/537.36",
            "x-requested-with": "XMLHttpRequest",
        }
        self.cookies = {
            "__gsas": "ID=eeb6a89775d353fc:T=1749577672:RT=1749577672:S=ALNI_MaIBex_3dHulhrGnFupNiyLCvPZsA",
            "cf_clearance": "UDM6zG5aQdnupSDm_pIrI0ahH3Qhf0C4OlzwVW3v9lc-1749711078-1.2.1.1-d6Cn4han6cxVdvXh8d2rCs94t44Ld6FDxpbHkRtKHfqp7kvMLSiNwJkjP_Y__l3tMmK4DA04X6WQG8hAav42mW6h3knRv6Tn_2XUgVAGngGmpizLslySbsm0riaopY33XFSRQ8nafhbavhdijYjzSI3JIbbsj1yywKSkFfloHhS0gPloasy7HFH8CNOlxF.lH7O7AsRJ7X5UMqJrmtk4LrAK4jskS5pp9vcOOIuo.hRqQ5NhlzFZklFpti22RVcmn1t463Z6DM8sAQbrlPObjp9qJYlaZo22uSHjLECc9uXeMy4cf_Xj8qjxPUTbOqpXmDM3qLHnC0563ZpCQHxLZBl2stE1Dr.gOaouTSAC_sw",
            "advanced_ads_visitor": '{"browser_width":1905}'
        }
        self.single_page_cate = single_page_cate

    def get_pages(self, book_id: int) -> list:
        """
        Retrieve all page URLs for a given book by its ID.

        Args:
            book_id (int): The ID of the book.

        Returns:
            list: List of page URLs.
        """
        url = "https://www.ebanglalibrary.com/wp-admin/admin-ajax.php"

        page_number = 1
        querystring = {"action": ["ld30_ajax_pager", "ld30_ajax_pager"], "pager_nonce": "26c3c2da05", "context": "course_content_shortcode",
                       "course_id": book_id, "shortcode_instance[course_id]": book_id, "shortcode_instance[num]": "100"}

        response = requests.get(url, headers=self.headers,
                                cookies=self.cookies, params=querystring)

        data = response.json()['data']['markup']
        html = HTMLParser(data)
        book_urls = [a.attributes.get('href', '')
                     for a in html.css('a.ld-item-name')]
        return book_urls

    def get_html_pages(self, book_url: str) -> list:
        """
        Retrieve all HTML page URLs for a given book URL.

        Args:
            book_url (str): The URL of the book's main page.

        Returns:
            list: List of page URLs.
        """
        response = requests.get(book_url, headers=self.headers)
        response.raise_for_status()  # Raise an error for bad responses
        html = HTMLParser(response.text)
        book_urls = [a.attributes.get('href', '')
                     for a in html.css('a.ld-item-name')]
        print(len(book_urls), "page urls found")
        return book_urls

    def get_cate_id(self, category_name: list) -> int:
        """
        Get the category ID for a given category name.

        Args:
            category_name (list): List of category name strings.

        Returns:
            int: Category ID.
        """
        with importlib.resources.files('bookspointer').joinpath('categories.json').open('r', encoding='utf-8') as file:
        # with open('bookspointer/bookspointer/categories.json', 'r', encoding='utf-8') as file:
            categories = json.load(file)
            category_name = ' '.join(category_name)
        for category in categories:
            if category['label'] == category_name:
                return category['id']
            elif 'ইতিহাস' in category_name:
                return 1
            elif 'কৌতুক' in category_name:
                return 2
            elif 'উপন্যাস' in category_name:
                return 3
            elif "থ্রিলার রহস্য রোমাঞ্চ অ্যাডভেঞ্চার" in category_name:
                return 4
            elif 'গল্পগ্রন্থ' in category_name:
                return 5
            elif 'গল্পের বই' in category_name:
                return 5
            elif "Editor's Choice" in category_name:
                return 5
            elif "ভ্রমণ কাহিনী" in category_name:
                return 6
            elif "বৈজ্ঞানিক কল্পকাহিনী" in category_name:
                return 9
            elif "ধর্ম ও দর্শন" in category_name:
                return 10
            elif "ইসলামিক বই" in category_name:
                return 10
            elif "ধর্মীয় বই" in category_name:
                return 10
            elif "সংস্কৃত" in category_name:
                return 10
            elif "কাব্যগ্রন্থ / কবিতা" in category_name:
                return 12
            elif "প্রবন্ধ ও গবেষণা" in category_name:
                return 13
            elif "রচনা" in category_name:
                return 13
            elif "কিশোর সাহিত্য" in category_name:
                return 14
            elif "আত্মজীবনী ও স্মৃতিকথা" in category_name:
                return 15
            elif "আত্মউন্নয়নমূলক বই" in category_name:
                return 15
            elif "নাটক" in category_name:
                return 16
            elif "গোয়েন্দা" in category_name:
                return 18
            elif "ভৌতিক" in category_name:
                return 19
            elif "হরর" in category_name:
                return 19
            elif "ভূতের বই" in category_name:
                return 19
            else:
                return 20
        return 20

    def get_book_list(self, author_url: str) -> list:
        """
        Scrape the list of books for a given author URL.

        Args:
            author_url (str): The URL of the author's page.

        Returns:
            list: List of dictionaries with book info (title, author, link).
        """
        books = []
        try:
            response = requests.get(author_url, headers=self.headers)
            html = HTMLParser(response.text)
            book_elements = html.css('article.entry-archive')
            print(f'{len(book_elements)} books found for the author.')
            for book in book_elements:
                title_ele = book.css_first(
                    'a.entry-title-link').text(strip=True)
                title = title_ele.split('–')[0].strip(
                ) if '–' in title_ele else title_ele
                author = title_ele.split('–')[1].strip(
                ) if '–' in title_ele else 'Unknown Author'
                book_link = book.css_first(
                    'a.entry-title-link').attributes.get('href', '')
                books.append({
                    'title': title,
                    'author': author,
                    'link': book_link
                })
            return books
        except requests.RequestException as e:
            print(f"An error occurred: {e}")
            return []

    def get_book_details(self, book_info: dict, author_id: int = 1) -> list:
        """
        Scrape detailed information for a given book.

        Args:
            book_info (dict): Dictionary with book info (title, author, link).
            author_id (int, optional): ID of the author. Defaults to 1.

        Returns:
            list: List of dictionaries with detailed book data.
        """
        books = []
        try:
            response = requests.get(book_info['link'], headers=self.headers)
            response.raise_for_status()  # Raise an error for bad responses
            html = HTMLParser(response.text)
            book_ele = html.css_first('h1.page-header-title').text(strip=True)
            title = book_ele.split('–')[0].strip(
            ) if '–' in book_ele else 'Unknown Title'
            if title == 'Unknown Title':
                title = html.css_first('h1.page-header-title').text(strip=True)
            category = [cat.text(strip=True) for cat in html.css(
                'span.entry-terms-ld_course_category a')]
            cate_id = self.get_cate_id(category_name=category)
            try:
                series = html.css_first(
                    'span.entry-terms-series a').text(strip=True)
            except:
                series = category
            paragraphs = html.css_first('div.ld-tabs-content div')
            button = paragraphs.css_first('button')
            if button:
                button.decompose()
            content_str = ''  # str(paragraphs.html) if paragraphs else ''
            contents = [content_str]
            book_id = html.css_first(
                'div.ld-tab-content').attributes.get('id').split('-')[-1]
            book = {
                'book_id': book_id,
                'title': title,
                'author': book_info['author'],
                'author_id': author_id,
                'category': category,
                'category_id': cate_id,
                'series': series,
                'content': '',
                'url': book_info['link']
            }
            all_books_url = self.get_html_pages(book_info['link'])
            for idx, book_url in enumerate(all_books_url, start=1):
                new_content = self.get_book_content(book_url)
                if new_content:
                    if cate_id in self.single_page_cate:
                        book_copy = book.copy()  # or use copy.deepcopy(book) if nested dicts
                        book_copy['title'] = new_content['title']
                        book_copy['content'] = new_content['content']
                        book_copy['url'] = book_url
                        print(book_copy)
                        books.append(book_copy)
                    else:
                        contents.append(new_content.get('content', ''))
                print(
                    f"Page Processed {idx}/{len(all_books_url)}: Author ID: {author_id}")
            book['content'] = '<br/>'.join(contents)
            books.append(book)
            return books
        except requests.RequestException as e:
            print(f"An error occurred while fetching book details: {e}")
            return []

    def get_book_content(self, url: str) -> dict:
        """
        Scrape the content and title from a book page URL.

        Args:
            url (str): The URL of the book page.

        Returns:
            dict: Dictionary with 'content' and 'title', or None if failed.
        """
        try:
            response = requests.get(url, headers=self.headers)
            response.raise_for_status()
            html = HTMLParser(response.text.replace('</p>', '</p><br/>'))
            content_div = html.css_first('div.ld-tabs-content > div')
            title = html.css_first('div.ld-focus-content h1').text(strip=True)
            for button in content_div.css('button'):
                button.decompose()
            if content_div:
                content_str = str(content_div.html)
                # print(content_str)
                return {'content': content_str, 'title': title}
        except requests.RequestException as e:
            print(f"An error occurred while fetching book content: {e}")
            return None



import requests
from rich import print
import pandas as pd
import gspread
from google.oauth2.service_account import Credentials
from bookspointer.server import AuthorAPI
import os
import json
from dotenv import load_dotenv
import importlib.resources


class AuthorSheetManager:
    def __init__(self, sheet_name="Books Pointer Data", worksheet_name="verified_authors", creds_file="Credentials.json"):
        self.sheet_name = sheet_name
        self.worksheet_name = worksheet_name
        self.creds_file = creds_file
        self.author_api = AuthorAPI()

        # Load .env and patch client_email if present
        load_dotenv()
        client_email = os.getenv("CLIENT_EMAIL")
        client_id = os.getenv("CLIENT_ID")
        private_key_id = os.getenv("PRIVATE_KEY_ID")

        with importlib.resources.files('bookspointer').joinpath(self.creds_file).open("r", encoding="utf-8") as f:
            creds_data = json.load(f)

        creds_data["client_email"] = client_email
        creds_data["client_id"] = client_id
        creds_data["private_key_id"] = private_key_id

        scopes = [
            'https://www.googleapis.com/auth/spreadsheets',
            'https://www.googleapis.com/auth/drive'
        ]
        creds = Credentials.from_service_account_info(
            creds_data, scopes=scopes)
        self.gc = gspread.authorize(creds)
        self.sh = self.gc.open(self.sheet_name)
        self.worksheet = self.sh.worksheet(self.worksheet_name)

    def get_authors(self, page=1):
        print(f"Getting authors from bookspointer")
        authors = []
        url = "https://api.bookspointer.com/authors"
        headers = {
            "Accept-Language": "en-US,en;q=0.9,bn;q=0.8",
            "Connection": "keep-alive",
            "Origin": "https://bookspointer.com",
            "Referer": "https://bookspointer.com/",
            "Sec-Fetch-Dest": "empty",
            "Sec-Fetch-Mode": "cors",
            "Sec-Fetch-Site": "same-site",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36",
            "accept": "application/json",
            "content-type": "application/json",
            "sec-ch-ua": '"Not)A;Brand";v="8", "Chromium";v="138", "Google Chrome";v="138"',
            "sec-ch-ua-mobile": "?0",
            "sec-ch-ua-platform": '"Windows"'
        }
        data = {
            "userId": 248,
            "device": 1,
            "page": page,
            "limit": 200
        }

        response = requests.post(url, headers=headers, json=data)
        data = response.json()['authors']
        if not data:
            print("No authors found.")
            return []
        for d in data:
            author = {
                'id': d['id'],
                'first_name': d['firstName'],
                'last_name': d['lastName'],
                'full_name': f'{d['firstName']} {d['lastName']}',
                'author_link': '=VLOOKUP(D254,table,3,0)',
                'is_scraped': False,
            }
            authors.append(author)
        return authors

    def save_authors_to_sheet(self, df):
        print(f"Saving authors to sheet")
        try:
            worksheet = self.worksheet
            # Get all existing ids from the sheet
            sheet_records = worksheet.get_all_records()
            existing_ids = set(str(row.get("id", "")) for row in sheet_records)

            # Filter out events whose event_id is already in the sheet
            new_rows = df[~df["id"].astype(str).isin(existing_ids)]

            if new_rows.empty:
                print(
                    "[bold yellow]No truly new events to add to Google Sheets.[/bold yellow]")
                return

            # Get the next empty row
            # +2 because sheet_records doesn't include header row
            next_row = len(sheet_records) + 2

            # Prepare data with formulas
            for index, row in new_rows.iterrows():
                row_data = [
                    row['id'],
                    row['first_name'],
                    row['last_name'],
                    row['full_name'],
                    # Formula with dynamic row reference
                    '=IFERROR(VLOOKUP(D' + str(next_row) + ',Authors,2,0), "")',
                    row['is_scraped']
                ]

                # Append the row with value_input_option to treat formulas properly
                worksheet.append_row(
                    row_data, value_input_option='USER_ENTERED')
                next_row += 1

            print(
                f"[bold green]Added {len(new_rows)} new authors to Google Sheet.[/bold green]")

        except Exception as e:
            if df.empty:
                print(
                    "[bold yellow]No new authors to add to Google Sheet.[/bold yellow]")
                return
            else:
                print(
                    f"[bold red]Failed to add Authors to Google Sheet: {e}[/bold red]")

    def get_unscraped_authors(self):
        """
        Get all authors from the sheet where is_scraped is FALSE
        Returns a list of dictionaries
        """
        try:
            worksheet = self.worksheet
            # Get all records from the sheet
            sheet_records = worksheet.get_all_records()

            # Filter records where is_scraped is FALSE
            unscraped_authors = []
            for record in sheet_records:
                # Check if is_scraped is False (boolean) or "FALSE" (string)
                is_scraped = record.get('is_scraped', False)
                if is_scraped == False or str(is_scraped).upper() == "FALSE":
                    unscraped_authors.append(record)

            print(
                f"[bold green]Found {len(unscraped_authors)} unscraped authors.[/bold green]")
            return unscraped_authors

        except Exception as e:
            print(f"[bold red]Failed to get unscraped authors: {e}[/bold red]")
            return []

    def update_scraped_status(self, author_id: str):
        """
        Update the is_scraped status from FALSE to TRUE for a given author ID
        """
        try:
            worksheet = self.worksheet
            # Get all records from the sheet
            sheet_records = worksheet.get_all_records()

            # Find the row with the matching author ID
            target_row = None
            # start=2 because row 1 is header
            for i, record in enumerate(sheet_records, start=2):
                if str(record.get('id', '')) == str(author_id):
                    target_row = i
                    break

            if target_row is None:
                print(
                    f"[bold red]Author with ID {author_id} not found in the sheet.[/bold red]")
                return False

            # Get the column index for 'is_scraped' column
            headers = list(sheet_records[0].keys())
            try:
                is_scraped_col_index = headers.index('is_scraped')
                is_scraped_col = is_scraped_col_index + 1  # Convert to 1-based index
            except ValueError:
                # If 'is_scraped' column not found, assume it's the last column
                is_scraped_col = len(headers)

            # Update the cell value to TRUE using update_cell method
            print(
                f"[bold blue]Updating cell at row {target_row}, column {is_scraped_col} for author ID {author_id}[/bold blue]")
            worksheet.update_cell(target_row, is_scraped_col, True)

            print(
                f"[bold green]Successfully updated author ID {author_id} is_scraped status to TRUE.[/bold green]")
            return True

        except Exception as e:
            print(f"[bold red]Failed to update author status: {e}[/bold red]")
            return False

    def run(self):
        authors = []
        for i in range(1, 100):
            try:
                new_authors = self.get_authors(i)
            except:
                new_authors = []

            if len(new_authors) == 0:
                break
            else:
                authors.extend(new_authors)

        if len(authors) >= 0:
            df = pd.DataFrame(authors)
            self.save_authors_to_sheet(df)

        unscraped_authors = self.get_unscraped_authors()
        if len(unscraped_authors) == 0:
            return
        for author in unscraped_authors:
            print(author)
            self.update_scraped_status(int(author['id']))
            self.author_api.create(author)


if __name__ == "__main__":
    manager = AuthorSheetManager().run()



import json
import requests
from bookspointer.server import BookAPI, TokenAPI

book_api = BookAPI()
token_api = TokenAPI()


class BookspointerAPI:
    def __init__(self, token):
        self.token = token
        self.url = 'https://api.bookspointer.com/admin/create-book'
        self.headers = {
            'Accept': '*/*',
            'Accept-Language': 'en-BD,en-US;q=0.9,en;q=0.8',
            'Connection': 'keep-alive',
            'Origin': 'https://bookspointer.com',
            'Referer': 'https://bookspointer.com/',
            'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/137.0.0.0 Safari/537.36',
            'authorization': f'Bearer {self.token}',
            'sec-ch-ua': '"Google Chrome";v="137", "Chromium";v="137", "Not/A)Brand";v="24"',
            'sec-ch-ua-mobile': '?0',
            'sec-ch-ua-platform': '"Windows"'
        }

    def post_book(self, book):
        if book['category'] == 'অসম্পূর্ণ বই':
            series = 'অসম্পূর্ণ বই'
        else:
            series = ''

        data_dict = {
            "title": book['title'],
            "category": {
                "id": book['category_id'],
            },
            "author": {
                "id": book['author_id'],
            },
            "content": book['content'],
            "tags": [],
            "seriesName": series,
        }

        files = {
            'data': (None, json.dumps(data_dict), 'application/json')
        }

        response = requests.post(self.url, headers=self.headers, files=files)
        print(book)
        book.pop('content')
        print(book)
        try:
            book_id = response.json()['last_book']['id']
            message = f"Book created with ID: {book_id}"
            update_book = book_api.update_book(book['book_id'], {'is_posted': True})
        except KeyError:
            message = response.json()['message']
            update_book = book_api.update_book(book['book_id'], {'is_posted': True})
        except Exception as e:
            message = f"Failed to post book {book['title']} : {e}"

        print("Response:", message)
